/*****************************************************************************
 * Open MCT Web, Copyright (c) 2014-2015, United States Government
 * as represented by the Administrator of the National Aeronautics and Space
 * Administration. All rights reserved.
 *
 * Open MCT Web is licensed under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * Open MCT Web includes source code licensed under additional open source
 * licenses. See the Open Source Licenses file (LICENSES.md) included with
 * this source code distribution or the Licensing information page available
 * at runtime from the About dialog for additional information.
 *****************************************************************************/
/*global define,Promise,describe,it,xdescribe,expect,beforeEach,waitsFor,jasmine*/

define(
    ["../../src/services/Instantiate"],
    function (Instantiate) {
        'use strict';

        describe("The 'instantiate' service", function () {

            var mockCapabilityService,
                mockCapabilityConstructor,
                mockCapabilityInstance,
                mockCapabilities,
                testModel,
                instantiate,
                domainObject;

            beforeEach(function () {
                mockCapabilityService = jasmine.createSpyObj(
                    'capabilityService',
                    ['getCapabilities']
                );
                mockCapabilityConstructor = jasmine.createSpy('capability');
                mockCapabilityInstance = {};
                mockCapabilityService.getCapabilities.andReturn({
                    something: mockCapabilityConstructor
                });
                mockCapabilityConstructor.andReturn(mockCapabilityInstance);

                testModel = { someKey: "some value" };

                instantiate = new Instantiate(mockCapabilityService);
                domainObject = instantiate(testModel);
            });

            it("loads capabilities from the capability service", function () {
                expect(mockCapabilityService.getCapabilities)
                    .toHaveBeenCalledWith(testModel);
            });

            it("exposes loaded capabilities from the created object", function () {
                expect(domainObject.getCapability('something'))
                    .toBe(mockCapabilityInstance);
                expect(mockCapabilityConstructor)
                    .toHaveBeenCalledWith(domainObject);
            });

            it("exposes the provided model", function () {
                expect(domainObject.getModel()).toEqual(testModel);
            });

            it("provides unique identifiers", function () {
                expect(domainObject.getId()).toEqual(jasmine.any(String));
                expect(instantiate(testModel).getId())
                    .not.toEqual(domainObject.getId());
            });
        });

    }
);
